@@ -83,7 +83,6 @@ import org.eclipse.ui.ISelectionListener;
 import org.eclipse.ui.IWorkbenchPart;
 import org.eclipse.ui.actions.ActionFactory;
 import org.eclipse.ui.actions.BaseSelectionListenerAction;
-import org.eclipse.ui.part.IPageSite;
 import org.eclipse.ui.part.Page;
 
 /**
@@ -151,7 +150,7 @@ final class EditViewPage extends Page implements ISelectionListener {
           else {
             editor = new TextCellEditor(viewer.getTree());
             editor.setValidator(CellEditorValidatorFacotory.createValidator(Primitives
-                    .getPrimitiveClass(value.getFeature())));
+                    .getPrimitiveClass(value.getFeatureStructure().getCAS().getTypeSystem(), value.getFeature())));
           }
 
           return editor;
@@ -231,7 +230,7 @@ final class EditViewPage extends Page implements ISelectionListener {
         // otherwise return boolean number
         if (!featureValue.getFeature().getRange().getName().equals(
                 CAS.TYPE_NAME_BOOLEAN)) {
-          return Primitives.getPrimitiv(featureValue.getFeatureStructure(), featureValue.getFeature()).toString();
+          return Primitives.getPrimitive(featureValue.getFeatureStructure(), featureValue.getFeature()).toString();
         }
         else {
           // for booleans
@@ -404,7 +403,7 @@ final class EditViewPage extends Page implements ISelectionListener {
             } else if (type.getName().equals(CAS.TYPE_NAME_FS_ARRAY)) {
               fs = document.getCAS().createArrayFS(arraySize);
             } else {
-              throw new CasEditorError("Unkown array type!");
+              throw new CasEditorError("Unkown array type: " + type.getName() + "!");
             }  
       }
       else if (ts.subsumes(ts.getType(CAS.TYPE_NAME_ANNOTATION), type)) {
@@ -597,7 +596,7 @@ final class EditViewPage extends Page implements ISelectionListener {
 
     valueColumn.setEditingSupport(new ValueEditingSupport(viewer));
 
-
+    
     FeatureStructureContentProvider contentProvider =
         new FeatureStructureContentProvider(document);
 
@@ -744,26 +743,21 @@ final class EditViewPage extends Page implements ISelectionListener {
 
   @Override
   public void setFocus() {
-  }
-
-  @Override
-  public void init(IPageSite pageSite) {
-    super.init(pageSite);
-
-    // pin action
-    pinAction = new PinAction();
-
-    pinAction.setText("Pin");
-    pinAction.setImageDescriptor(CasEditorPlugin.getTaeImageDescriptor(Images.PIN));
-    pageSite.getActionBars().getToolBarManager().add(pinAction);
-
+    viewer.getControl().setFocus();
   }
 
   @Override
   public void setActionBars(IActionBars actionBars) {
+    // pin action
+    pinAction = new PinAction();
+    
+    pinAction.setText("Pin");
+    pinAction.setImageDescriptor(CasEditorPlugin.getTaeImageDescriptor(Images.PIN));
+    actionBars.getToolBarManager().add(pinAction);
+    
     CreateFeatureStructrueValue createAction = new CreateFeatureStructrueValue();
     createAction.setImageDescriptor(CasEditorPlugin.getTaeImageDescriptor(Images.ADD));
-    getSite().getActionBars().getToolBarManager().add(createAction);
+    actionBars.getToolBarManager().add(createAction);
 
     // TODO: setActionBars is depreciated, but registration of change listener
     // does not work in init method
@@ -771,26 +765,28 @@ final class EditViewPage extends Page implements ISelectionListener {
 
     // delete action
     DeleteFeatureStructureValue deleteAction = new DeleteFeatureStructureValue();
-    getSite().getActionBars().setGlobalActionHandler(ActionFactory.DELETE.getId(), deleteAction);
+    actionBars.setGlobalActionHandler(ActionFactory.DELETE.getId(), deleteAction);
 
     getSite().getSelectionProvider().addSelectionChangedListener(deleteAction);
 
-    getSite().getActionBars().getToolBarManager().add(
+    actionBars.getToolBarManager().add(
             ActionFactory.DELETE.create(getSite().getWorkbenchWindow()));
   }
 
   public void selectionChanged(IWorkbenchPart part, ISelection selection) {
 
-    if (selection instanceof IStructuredSelection) {
-
-      FeatureStructureSelection fsSelection = new FeatureStructureSelection(
-              (IStructuredSelection) selection);
-
-      if (fsSelection.size() == 1 && !pinAction.isChecked()) {
-
-        // filter out selection which are cause by this view itself
-        if (editView != part) {
-          viewer.setInput(fsSelection.toList().get(0));
+    if (getSite().getPage().getActiveEditor() == editor) {
+      if (selection instanceof IStructuredSelection) {
+  
+        FeatureStructureSelection fsSelection = new FeatureStructureSelection(
+                (IStructuredSelection) selection);
+  
+        if (fsSelection.size() == 1 && !pinAction.isChecked()) {
+  
+          // filter out selection which are cause by this view itself
+          if (editView != part) {
+            viewer.setInput(fsSelection.toList().get(0));
+          }
         }
       }
     }
@@ -799,5 +795,6 @@ final class EditViewPage extends Page implements ISelectionListener {
   @Override
   public void dispose() {
     getSite().getPage().removeSelectionListener(this);
+    super.dispose();
   }
 }
\ No newline at end of file
